library IEEE;
use IEEE.std_logic_1164.all;
use work.n7.all;

entity testUART is
  
end testUART;

architecture testUART_arch of testUART is

  component UARTunit 
    port (
      clk, reset : in  std_logic;
      cs, rd, wr : in  std_logic;
      RxD        : in  std_logic;
      TxD        : out std_logic;
      IntR       : out std_logic;         -- interruption de reception
      IntT       : out std_logic;         -- interruption d'emission
      addr       : in  std_logic_vector(1 downto 0);
      data_in    : in  std_logic_vector(7 downto 0);
      data_out   : out std_logic_vector(7 downto 0));
  end component;

  signal h, rst : std_logic;
  signal h1, rst1 : std_logic;
  
  -- UART0
  signal cs0, rd0, wr0 : std_logic;
  signal RxD0, TxD0 : std_logic;
  signal IntR0, IntT0 : std_logic;
  signal addr0 : std_logic_vector(1 downto 0);
  signal data_in0, data_out0 : std_logic_vector(7 downto 0);

  -- UART1
  signal cs1, rd1, wr1 : std_logic;
  signal RxD1, TxD1 : std_logic;
  signal IntR1, IntT1 : std_logic;
  signal addr1 : std_logic_vector(1 downto 0);
  signal data_in1, data_out1 : std_logic_vector(7 downto 0);

  constant periode : time := 6400 ns;
  constant decalage : time := 2*periode;
  constant delai_transmit : time := 100 us;
  
begin  -- testUART_arch

  C: clock(h, periode, 0 ns, periode*1000);
  C1: clock(h1, periode, 0 ns, periode*1000);
  
  U0: UARTunit port map (h, rst, cs0, rd0, wr0, RxD0, TxD0, IntR0, IntT0, addr0, data_in0, data_out0);

  U1: UARTunit port map (h1, rst1, cs1, rd1, wr1, RxD1, TxD1, IntR1, IntT1, addr1, data_in1, data_out1);

  RxD0<=TxD1;
  RxD1<=TxD0 after delai_transmit;
  
  rst<='0', '1' after periode;
  rst1<='0', '1' after periode;
  
  p0: process
  begin  -- processus qui simule un processeur qui emet
    cs0<='1';
    wr0<='1';
    rd0<='1';
    addr0<="00";
    wait until rst='1';
    wait for periode*3;
    
    -- Interruption pour la transmission : le processeur peut transmettre 
    if IntT0='1' then
      wait until IntT0='0';
    end if;
    
    -- on demande l'emission du message 0xAA 
    wr0<='0';
    rd0<='1';
    addr0<="00";
    data_in0<="10101010";

    if IntT0='0' then
      wait until IntT0='1';    
    end if;
    cs0<='1';
    wr0<='1';
    rd0<='1';
    addr0<="00";
    data_in0<="11111111";
    
    
    -- lorsque l'UART est libre a nouveau, on peut emettre un autre 
    -- caractere. Ici 0x55
    if IntT0='1' then
      wait until IntT0='0';     
    end if;   
    cs0<='0';
    wr0<='0';
    rd0<='1';
    addr0<="00";
    data_in0<="01010101";
    
    if IntT0='0' then
      wait until IntT0='1';
    end if;
    cs0<='1';
    wr0<='1';
    rd0<='1';
    addr0<="00";
    
    -- lorsque l'UART est libre a nouveau, on peut emettre un autre 
    -- caractere. Ici 0x24
    if IntT0='1' then
      wait until IntT0='0';     
    end if;   
    cs0<='0';
    wr0<='0';
    rd0<='1';
    addr0<="00";
    data_in0<="00100100";
    
    if IntT0='0' then
      wait until IntT0='1';
    end if;
    cs0<='1';
    wr0<='1';
    rd0<='1';
    addr0<="00";
    wait;
    
  end process;

  
   p1: process
   begin  -- processus qui simule un processeur qui recoit
    cs1<='1';
    wr1<='1';
    rd1<='1';
    addr1<="00";
    data_in1<="11111111";
    wait for periode*65;
    
    -- on teste le contenu de ctrlReg
    cs1<='0';
    wr1<='1';
    rd1<='0';
    addr1<="01";
    data_in1<="11111111";
    wait for periode*5;
    
    cs1<='1';
    wr1<='1';
    rd1<='1';
    addr1<="00";
    data_in1<="11111111";
    
    -- on attend la reception d'un caractere, i.e. reception d'une 
    -- interruption
    if IntR1='1' then 
    	wait until IntR1='0';
    end if;
    wait until h1='1' and h1'event;
    
    -- on lit le resultat
    cs1<='0';
    rd1<='0';
    wr1<='1';
    addr1<="00";
    --if IntR1='0' then 
    --	wait until IntR1='1';
    --end if;
    wait until h1='1' and h1'event;
    
    -- on recommence la reception
    cs1<='1';
    rd1<='1';
    wr1<='1';
    addr1<="00";
    
    --reception d'une interruption
    if IntR1='1' then 
    	wait until IntR1='0';
    end if;
    wait until h1='1' and h1'event;
    
    -- lecture du resultat
    cs1<='0';
    rd1<='0';
    wr1<='1';
    addr1<="00";
    wait until h1='1' and h1'event;
    cs1<='1';
    rd1<='1';
    wr1<='1';
    addr1<="00";
    
    -- on recommence la reception
    cs1<='1';
    rd1<='1';
    wr1<='1';
    addr1<="00";
    
    --reception d'une interruption
    if IntR1='1' then 
    	wait until IntR1='0';
    end if;
    wait until h1='1' and h1'event;
    
    -- lecture du resultat
    cs1<='0';
    rd1<='0';
    wr1<='1';
    addr1<="00";
    wait until h1='1' and h1'event;
    cs1<='1';
    rd1<='1';
    wr1<='1';
    addr1<="00";

    -- on recommence la reception
    cs1<='1';
    rd1<='1';
    wr1<='1';
    addr1<="00";
    
    --reception d'une interruption
    if IntR1='1' then 
    	wait until IntR1='0';
    end if;
    wait until h1='1' and h1'event;
    
    -- lecture du resultat
    cs1<='0';
    rd1<='0';
    wr1<='1';
    addr1<="00";
    wait until h1='1' and h1'event;
    cs1<='1';
    rd1<='1';
    wr1<='1';
    addr1<="00";

    wait;
    
  end process;
  
end testUART_arch;
